@@ -1,7 +1,7 @@
 <?php
 /*
 ** Zabbix
-** Copyright (C) 2001-2017 Zabbix SIA
+** Copyright (C) 2001-2021 Zabbix SIA
 **
 ** This program is free software; you can redistribute it and/or modify
 ** it under the terms of the GNU General Public License as published by
@@ -24,7 +24,17 @@
  */
 class CHostGroup extends CApiService {
 
-	protected $tableName = 'groups';
+	public const ACCESS_RULES = [
+		'get' => ['min_user_type' => USER_TYPE_ZABBIX_USER],
+		'create' => ['min_user_type' => USER_TYPE_SUPER_ADMIN],
+		'update' => ['min_user_type' => USER_TYPE_ZABBIX_ADMIN],
+		'delete' => ['min_user_type' => USER_TYPE_ZABBIX_ADMIN],
+		'massadd' => ['min_user_type' => USER_TYPE_ZABBIX_ADMIN],
+		'massupdate' => ['min_user_type' => USER_TYPE_ZABBIX_ADMIN],
+		'massremove' => ['min_user_type' => USER_TYPE_ZABBIX_ADMIN]
+	];
+
+	protected $tableName = 'hstgrp';
 	protected $tableAlias = 'g';
 	protected $sortColumns = ['groupid', 'name'];
 
@@ -37,67 +47,67 @@ class CHostGroup extends CApiService {
 	 */
 	public function get($params) {
 		$result = [];
-		$userType = self::$userData['type'];
-		$userid = self::$userData['userid'];
 
 		$sqlParts = [
-			'select'	=> ['groups' => 'g.groupid'],
-			'from'		=> ['groups' => 'groups g'],
+			'select'	=> ['hstgrp' => 'g.groupid'],
+			'from'		=> ['hstgrp' => 'hstgrp g'],
 			'where'		=> [],
 			'order'		=> [],
 			'limit'		=> null
 		];
 
 		$defOptions = [
-			'groupids'					=> null,
-			'hostids'					=> null,
-			'templateids'				=> null,
-			'graphids'					=> null,
-			'triggerids'				=> null,
-			'maintenanceids'			=> null,
-			'monitored_hosts'			=> null,
-			'templated_hosts'			=> null,
-			'real_hosts'				=> null,
-			'with_hosts_and_templates'	=> null,
-			'with_items'				=> null,
-			'with_simple_graph_items'	=> null,
-			'with_monitored_items'		=> null,
-			'with_triggers'				=> null,
-			'with_monitored_triggers'	=> null,
-			'with_httptests'			=> null,
-			'with_monitored_httptests'	=> null,
-			'with_graphs'				=> null,
-			'with_applications'			=> null,
-			'editable'					=> null,
-			'nopermissions'				=> null,
+			'groupids'								=> null,
+			'hostids'								=> null,
+			'templateids'							=> null,
+			'graphids'								=> null,
+			'triggerids'							=> null,
+			'maintenanceids'						=> null,
+			'monitored_hosts'						=> null,
+			'templated_hosts'						=> null,
+			'real_hosts'							=> null,
+			'with_hosts_and_templates'				=> null,
+			'with_items'							=> null,
+			'with_item_prototypes'					=> null,
+			'with_simple_graph_items'				=> null,
+			'with_simple_graph_item_prototypes'		=> null,
+			'with_monitored_items'					=> null,
+			'with_triggers'							=> null,
+			'with_monitored_triggers'				=> null,
+			'with_httptests'						=> null,
+			'with_monitored_httptests'				=> null,
+			'with_graphs'							=> null,
+			'with_graph_prototypes'					=> null,
+			'with_applications'						=> null,
+			'editable'								=> false,
+			'nopermissions'							=> null,
 			// filter
-			'filter'					=> null,
-			'search'					=> null,
-			'searchByAny'				=> null,
-			'startSearch'				=> false,
-			'excludeSearch'				=> false,
-			'searchWildcardsEnabled'	=> null,
+			'filter'								=> null,
+			'search'								=> null,
+			'searchByAny'							=> null,
+			'startSearch'							=> false,
+			'excludeSearch'							=> false,
+			'searchWildcardsEnabled'				=> null,
 			// output
-			'output'					=> API_OUTPUT_EXTEND,
-			'selectHosts'				=> null,
-			'selectTemplates'			=> null,
-			'selectGroupDiscovery'		=> null,
-			'selectDiscoveryRule'		=> null,
-			'countOutput'				=> false,
-			'groupCount'				=> false,
-			'preservekeys'				=> false,
-			'sortfield'					=> '',
-			'sortorder'					=> '',
-			'limit'						=> null,
-			'limitSelects'				=> null
+			'output'								=> API_OUTPUT_EXTEND,
+			'selectHosts'							=> null,
+			'selectTemplates'						=> null,
+			'selectGroupDiscovery'					=> null,
+			'selectDiscoveryRule'					=> null,
+			'countOutput'							=> false,
+			'groupCount'							=> false,
+			'preservekeys'							=> false,
+			'sortfield'								=> '',
+			'sortorder'								=> '',
+			'limit'									=> null,
+			'limitSelects'							=> null
 		];
 		$options = zbx_array_merge($defOptions, $params);
 
 		// editable + PERMISSION CHECK
-		if ($userType != USER_TYPE_SUPER_ADMIN && !$options['nopermissions']) {
+		if (self::$userData['type'] != USER_TYPE_SUPER_ADMIN && !$options['nopermissions']) {
 			$permission = $options['editable'] ? PERM_READ_WRITE : PERM_READ;
-
-			$userGroups = getUserGroupsByUserId($userid);
+			$userGroups = getUserGroupsByUserId(self::$userData['userid']);
 
 			$sqlParts['where'][] = 'EXISTS ('.
 				'SELECT NULL'.
@@ -173,32 +183,34 @@ class CHostGroup extends CApiService {
 			$sqlParts['where']['hmh'] = 'g.groupid=mg.groupid';
 		}
 
-		$sub_sql_parts = [];
+		$sub_sql_common = [];
 
 		// monitored_hosts, real_hosts, templated_hosts, with_hosts_and_templates
 		if ($options['monitored_hosts'] !== null) {
-			$sub_sql_parts['from']['h'] = 'hosts h';
-			$sub_sql_parts['where']['hg-h'] = 'hg.hostid=h.hostid';
-			$sub_sql_parts['where'][] = dbConditionInt('h.status', [HOST_STATUS_MONITORED]);
+			$sub_sql_common['from']['h'] = 'hosts h';
+			$sub_sql_common['where']['hg-h'] = 'hg.hostid=h.hostid';
+			$sub_sql_common['where'][] = dbConditionInt('h.status', [HOST_STATUS_MONITORED]);
 		}
 		elseif ($options['real_hosts'] !== null) {
-			$sub_sql_parts['from']['h'] = 'hosts h';
-			$sub_sql_parts['where']['hg-h'] = 'hg.hostid=h.hostid';
-			$sub_sql_parts['where'][] = dbConditionInt('h.status', [HOST_STATUS_MONITORED, HOST_STATUS_NOT_MONITORED]);
+			$sub_sql_common['from']['h'] = 'hosts h';
+			$sub_sql_common['where']['hg-h'] = 'hg.hostid=h.hostid';
+			$sub_sql_common['where'][] = dbConditionInt('h.status', [HOST_STATUS_MONITORED, HOST_STATUS_NOT_MONITORED]);
 		}
 		elseif ($options['templated_hosts'] !== null) {
-			$sub_sql_parts['from']['h'] = 'hosts h';
-			$sub_sql_parts['where']['hg-h'] = 'hg.hostid=h.hostid';
-			$sub_sql_parts['where'][] = dbConditionInt('h.status', [HOST_STATUS_TEMPLATE]);
+			$sub_sql_common['from']['h'] = 'hosts h';
+			$sub_sql_common['where']['hg-h'] = 'hg.hostid=h.hostid';
+			$sub_sql_common['where'][] = dbConditionInt('h.status', [HOST_STATUS_TEMPLATE]);
 		}
 		elseif ($options['with_hosts_and_templates'] !== null) {
-			$sub_sql_parts['from']['h'] = 'hosts h';
-			$sub_sql_parts['where']['hg-h'] = 'hg.hostid=h.hostid';
-			$sub_sql_parts['where'][] = dbConditionInt('h.status',
+			$sub_sql_common['from']['h'] = 'hosts h';
+			$sub_sql_common['where']['hg-h'] = 'hg.hostid=h.hostid';
+			$sub_sql_common['where'][] = dbConditionInt('h.status',
 				[HOST_STATUS_MONITORED, HOST_STATUS_NOT_MONITORED, HOST_STATUS_TEMPLATE]
 			);
 		}
 
+		$sub_sql_parts = $sub_sql_common;
+
 		// with_items, with_monitored_items, with_simple_graph_items
 		if ($options['with_items'] !== null) {
 			$sub_sql_parts['from']['i'] = 'items i';
@@ -298,14 +310,52 @@ class CHostGroup extends CApiService {
 			')';
 		}
 
+		$sub_sql_parts = $sub_sql_common;
+
+		// with_item_prototypes, with_simple_graph_item_prototypes
+		if ($options['with_item_prototypes'] !== null) {
+			$sub_sql_parts['from']['i'] = 'items i';
+			$sub_sql_parts['where']['hg-i'] = 'hg.hostid=i.hostid';
+			$sub_sql_parts['where'][] = dbConditionInt('i.flags', [ZBX_FLAG_DISCOVERY_PROTOTYPE]);
+		}
+		elseif ($options['with_simple_graph_item_prototypes'] !== null) {
+			$sub_sql_parts['from']['i'] = 'items i';
+			$sub_sql_parts['where']['hg-i'] = 'hg.hostid=i.hostid';
+			$sub_sql_parts['where'][] = dbConditionInt('i.value_type', [ITEM_VALUE_TYPE_FLOAT, ITEM_VALUE_TYPE_UINT64]);
+			$sub_sql_parts['where'][] = dbConditionInt('i.status', [ITEM_STATUS_ACTIVE]);
+			$sub_sql_parts['where'][] = dbConditionInt('i.flags', [ZBX_FLAG_DISCOVERY_PROTOTYPE]);
+		}
+
+		// with_graph_prototypes
+		if ($options['with_graph_prototypes'] !== null) {
+			$sub_sql_parts['from']['i'] = 'items i';
+			$sub_sql_parts['from']['gi'] = 'graphs_items gi';
+			$sub_sql_parts['from']['gr'] = 'graphs gr';
+			$sub_sql_parts['where']['hg-i'] = 'hg.hostid=i.hostid';
+			$sub_sql_parts['where']['i-gi'] = 'i.itemid=gi.itemid';
+			$sub_sql_parts['where']['gi-gr'] = 'gi.graphid=gr.graphid';
+			$sub_sql_parts['where'][] = dbConditionInt('gr.flags', [ZBX_FLAG_DISCOVERY_PROTOTYPE]);
+		}
+
+		if ($sub_sql_parts) {
+			$sub_sql_parts['from']['hg'] = 'hosts_groups hg';
+			$sub_sql_parts['where']['g-hg'] = 'g.groupid=hg.groupid';
+
+			$sqlParts['where'][] = 'EXISTS ('.
+				'SELECT NULL'.
+				' FROM '.implode(',', $sub_sql_parts['from']).
+				' WHERE '.implode(' AND ', array_unique($sub_sql_parts['where'])).
+			')';
+		}
+
 		// filter
 		if (is_array($options['filter'])) {
-			$this->dbFilter('groups g', $options, $sqlParts);
+			$this->dbFilter('hstgrp g', $options, $sqlParts);
 		}
 
 		// search
 		if (is_array($options['search'])) {
-			zbx_db_search('groups g', $options, $sqlParts);
+			zbx_db_search('hstgrp g', $options, $sqlParts);
 		}
 
 		// limit
@@ -315,7 +365,7 @@ class CHostGroup extends CApiService {
 
 		$sqlParts = $this->applyQueryOutputOptions($this->tableName(), $this->tableAlias(), $options, $sqlParts);
 		$sqlParts = $this->applyQuerySortOptions($this->tableName(), $this->tableAlias(), $options, $sqlParts);
-		$res = DBselect($this->createSelectQueryFromParts($sqlParts), $sqlParts['limit']);
+		$res = DBselect(self::createSelectQueryFromParts($sqlParts), $sqlParts['limit']);
 		while ($group = DBfetch($res)) {
 			if ($options['countOutput']) {
 				if ($options['groupCount']) {
@@ -342,6 +392,7 @@ class CHostGroup extends CApiService {
 		if (!$options['preservekeys']) {
 			$result = zbx_cleanHashes($result);
 		}
+
 		return $result;
 	}
 
@@ -351,30 +402,45 @@ class CHostGroup extends CApiService {
 	 * @param array  $groups
 	 * @param string $groups[]['groupid']
 	 * @param string $groups[]['name']
-	 *
-	 * @return array
 	 */
 	private function inheritRights(array $groups) {
 		$parent_names = [];
 
 		foreach ($groups as $group) {
-			if (($pos = strrpos($group['name'], '/')) === false) {
-				continue;
-			}
+			$name = $group['name'];
 
-			$parent_names[substr($group['name'], 0, $pos)][] = $group['groupid'];
+			while (($pos = strrpos($name, '/')) !== false) {
+				$name = substr($name, 0, $pos);
+				$parent_names[$name][] = $group['groupid'];
+			}
 		}
 
 		if ($parent_names) {
-			$db_parent_groups = DB::select('groups', [
+			$options = [
 				'output' => ['groupid', 'name'],
 				'filter' => ['name' => array_keys($parent_names)]
-			]);
+			];
+			$result = DBselect(DB::makeSql('hstgrp', $options));
+
+			$db_parent_groups = [];
+
+			while ($row = DBfetch($result)) {
+				$db_parent_groups[$row['name']] = $row['groupid'];
+			}
 
 			$parent_groupids = [];
 
-			foreach ($db_parent_groups as $db_parent_group) {
-				$parent_groupids[$db_parent_group['groupid']] = $parent_names[$db_parent_group['name']];
+			foreach ($groups as $group) {
+				$name = $group['name'];
+
+				while (($pos = strrpos($name, '/')) !== false) {
+					$name = substr($name, 0, $pos);
+
+					if (array_key_exists($name, $db_parent_groups)) {
+						$parent_groupids[$db_parent_groups[$name]][] = $group['groupid'];
+						break;
+					}
+				}
 			}
 
 			if ($parent_groupids) {
@@ -400,6 +466,77 @@ class CHostGroup extends CApiService {
 		}
 	}
 
+	/**
+	 * Inherit tag filters from parent host groups.
+	 *
+	 * @param array  $groups
+	 * @param string $groups[]['groupid']
+	 * @param string $groups[]['name']
+	 */
+	private function inheritTagFilters(array $groups) {
+		$parent_names = [];
+
+		foreach ($groups as $group) {
+			$name = $group['name'];
+
+			while (($pos = strrpos($name, '/')) !== false) {
+				$name = substr($name, 0, $pos);
+				$parent_names[$name][] = $group['groupid'];
+			}
+		}
+
+		if ($parent_names) {
+			$options = [
+				'output' => ['groupid', 'name'],
+				'filter' => ['name' => array_keys($parent_names)]
+			];
+			$result = DBselect(DB::makeSql('hstgrp', $options));
+
+			$db_parent_groups = [];
+
+			while ($row = DBfetch($result)) {
+				$db_parent_groups[$row['name']] = $row['groupid'];
+			}
+
+			$parent_groupids = [];
+
+			foreach ($groups as $group) {
+				$name = $group['name'];
+
+				while (($pos = strrpos($name, '/')) !== false) {
+					$name = substr($name, 0, $pos);
+
+					if (array_key_exists($name, $db_parent_groups)) {
+						$parent_groupids[$db_parent_groups[$name]][] = $group['groupid'];
+						break;
+					}
+				}
+			}
+
+			if ($parent_groupids) {
+				$db_tag_filters = DB::select('tag_filter', [
+					'output' => ['usrgrpid', 'groupid', 'tag', 'value'],
+					'filter' => ['groupid' => array_keys($parent_groupids)]
+				]);
+
+				$tag_filters = [];
+
+				foreach ($db_tag_filters as $db_tag_filter) {
+					foreach ($parent_groupids[$db_tag_filter['groupid']] as $groupid) {
+						$tag_filters[] = [
+							'usrgrpid' => $db_tag_filter['usrgrpid'],
+							'groupid' => $groupid,
+							'tag' => $db_tag_filter['tag'],
+							'value' => $db_tag_filter['value']
+						];
+					}
+				}
+
+				DB::insertBatch('tag_filter', $tag_filters);
+			}
+		}
+	}
+
 	/**
 	 * @param array  $groups
 	 *
@@ -408,7 +545,7 @@ class CHostGroup extends CApiService {
 	public function create(array $groups) {
 		$this->validateCreate($groups);
 
-		$groupids = DB::insertBatch('groups', $groups);
+		$groupids = DB::insertBatch('hstgrp', $groups);
 
 		foreach ($groups as $index => &$group) {
 			$group['groupid'] = $groupids[$index];
@@ -416,6 +553,7 @@ class CHostGroup extends CApiService {
 		unset($group);
 
 		$this->inheritRights($groups);
+		$this->inheritTagFilters($groups);
 
 		$this->addAuditBulk(AUDIT_ACTION_ADD, AUDIT_RESOURCE_HOST_GROUP, $groups);
 
@@ -443,7 +581,7 @@ class CHostGroup extends CApiService {
 			}
 		}
 
-		DB::update('groups', $upd_groups);
+		DB::update('hstgrp', $upd_groups);
 
 		$this->addAuditBulk(AUDIT_ACTION_UPDATE, AUDIT_RESOURCE_HOST_GROUP, $groups, $db_groups);
 
@@ -454,9 +592,173 @@ class CHostGroup extends CApiService {
 	 * @param array $groupids
 	 * @param bool 	$nopermissions
 	 *
+	 * @throws APIException if the input is invalid.
+	 *
 	 * @return array
 	 */
 	public function delete(array $groupids, $nopermissions = false) {
+		$this->validateDelete($groupids, $db_groups, $nopermissions);
+
+		// delete screens items
+		$resources = [
+			SCREEN_RESOURCE_HOSTGROUP_TRIGGERS,
+			SCREEN_RESOURCE_HOST_INFO,
+			SCREEN_RESOURCE_TRIGGER_INFO,
+			SCREEN_RESOURCE_TRIGGER_OVERVIEW,
+			SCREEN_RESOURCE_DATA_OVERVIEW
+		];
+		DB::delete('screens_items', [
+			'resourceid' => $groupids,
+			'resourcetype' => $resources
+		]);
+
+		// delete sysmap element
+		if (!empty($groupids)) {
+			DB::delete('sysmaps_elements', ['elementtype' => SYSMAP_ELEMENT_TYPE_HOST_GROUP, 'elementid' => $groupids]);
+		}
+
+		// disable actions
+		// actions from conditions
+		$actionids = [];
+		$db_actions = DBselect(
+			'SELECT DISTINCT c.actionid'.
+			' FROM conditions c'.
+			' WHERE c.conditiontype='.CONDITION_TYPE_HOST_GROUP.
+				' AND '.dbConditionString('c.value', $groupids)
+		);
+		while ($db_action = DBfetch($db_actions)) {
+			$actionids[$db_action['actionid']] = $db_action['actionid'];
+		}
+
+		// actions from operations
+		$db_actions = DBselect(
+			'SELECT o.actionid'.
+			' FROM operations o,opgroup og'.
+			' WHERE o.operationid=og.operationid AND '.dbConditionInt('og.groupid', $groupids).
+			' UNION'.
+			' SELECT o.actionid'.
+			' FROM operations o,opcommand_grp ocg'.
+			' WHERE o.operationid=ocg.operationid AND '.dbConditionInt('ocg.groupid', $groupids)
+		);
+		while ($db_action = DBfetch($db_actions)) {
+			$actionids[$db_action['actionid']] = $db_action['actionid'];
+		}
+
+		if (!empty($actionids)) {
+			$update = [];
+			$update[] = [
+				'values' => ['status' => ACTION_STATUS_DISABLED],
+				'where' => ['actionid' => $actionids]
+			];
+			DB::update('actions', $update);
+		}
+
+		// delete action conditions
+		DB::delete('conditions', [
+			'conditiontype' => CONDITION_TYPE_HOST_GROUP,
+			'value' => $groupids
+		]);
+
+		// delete action operation groups
+		$operationids = [];
+		$db_operations = DBselect(
+			'SELECT DISTINCT og.operationid'.
+			' FROM opgroup og'.
+			' WHERE '.dbConditionInt('og.groupid', $groupids)
+		);
+		while ($db_operation = DBfetch($db_operations)) {
+			$operationids[$db_operation['operationid']] = $db_operation['operationid'];
+		}
+		DB::delete('opgroup', [
+			'groupid' => $groupids
+		]);
+
+		// delete action operation commands
+		$db_operations = DBselect(
+			'SELECT DISTINCT ocg.operationid'.
+			' FROM opcommand_grp ocg'.
+			' WHERE '.dbConditionInt('ocg.groupid', $groupids)
+		);
+		while ($db_operation = DBfetch($db_operations)) {
+			$operationids[$db_operation['operationid']] = $db_operation['operationid'];
+		}
+		DB::delete('opcommand_grp', [
+			'groupid' => $groupids
+		]);
+
+		// delete empty operations
+		$del_operationids = [];
+		$db_operations = DBselect(
+			'SELECT DISTINCT o.operationid'.
+			' FROM operations o'.
+			' WHERE '.dbConditionInt('o.operationid', $operationids).
+				' AND NOT EXISTS (SELECT NULL FROM opgroup og WHERE o.operationid=og.operationid)'.
+				' AND NOT EXISTS (SELECT NULL FROM opcommand_grp ocg WHERE o.operationid=ocg.operationid)'
+		);
+		while ($db_operation = DBfetch($db_operations)) {
+			$del_operationids[$db_operation['operationid']] = $db_operation['operationid'];
+		}
+
+		DB::delete('operations', ['operationid' => $del_operationids]);
+
+		DB::delete('hstgrp', ['groupid' => $groupids]);
+
+		$this->addAuditBulk(AUDIT_ACTION_DELETE, AUDIT_RESOURCE_HOST_GROUP, $db_groups);
+
+		return ['groupids' => $groupids];
+	}
+
+	/**
+	 * Check for duplicated host groups.
+	 *
+	 * @param array  $names
+	 *
+	 * @throws APIException  if host group already exists.
+	 */
+	private function checkDuplicates(array $names) {
+		$db_groups = DB::select('hstgrp', [
+			'output' => ['name'],
+			'filter' => ['name' => $names],
+			'limit' => 1
+		]);
+
+		if ($db_groups) {
+			self::exception(ZBX_API_ERROR_PARAMETERS,
+				_s('Host group "%1$s" already exists.', $db_groups[0]['name'])
+			);
+		}
+	}
+
+	/**
+	 * @param array $groups
+	 *
+	 * @throws APIException if the input is invalid.
+	 */
+	protected function validateCreate(array &$groups) {
+		if (self::$userData['type'] != USER_TYPE_SUPER_ADMIN) {
+			self::exception(ZBX_API_ERROR_PERMISSIONS, _('Only Super Admins can create host groups.'));
+		}
+
+		$api_input_rules = ['type' => API_OBJECTS, 'flags' => API_NOT_EMPTY | API_NORMALIZE, 'uniq' => [['name']], 'fields' => [
+			'name' =>	['type' => API_HG_NAME, 'flags' => API_REQUIRED, 'length' => DB::getFieldLength('hstgrp', 'name')]
+		]];
+		if (!CApiInputValidator::validate($api_input_rules, $groups, '/', $error)) {
+			self::exception(ZBX_API_ERROR_PARAMETERS, $error);
+		}
+
+		$this->checkDuplicates(zbx_objectValues($groups, 'name'));
+	}
+
+	/**
+	 * Validates if groups can be deleted.
+	 *
+	 * @param array $groupids
+	 * @param array $db_groups
+	 * @param bool $nopermissions
+	 *
+	 * @throws APIException if the input is invalid.
+	 */
+	private function validateDelete(array $groupids, array &$db_groups = null, $nopermissions) {
 		$api_input_rules = ['type' => API_IDS, 'flags' => API_NOT_EMPTY, 'uniq' => true];
 		if (!CApiInputValidator::validate($api_input_rules, $groupids, '/', $error)) {
 			self::exception(ZBX_API_ERROR_PARAMETERS, $error);
@@ -486,16 +788,16 @@ class CHostGroup extends CApiService {
 		}
 
 		// check if a group is used in a group prototype
-		$groupPrototype = DBFetch(DBselect(
+		$group_prototype = DBFetch(DBselect(
 			'SELECT groupid'.
 			' FROM group_prototype gp'.
 			' WHERE '.dbConditionInt('groupid', $groupids),
 			1
 		));
-		if ($groupPrototype) {
+		if ($group_prototype) {
 			self::exception(ZBX_API_ERROR_PARAMETERS,
 				_s('Group "%1$s" cannot be deleted, because it is used by a host prototype.',
-					$db_groups[$groupPrototype['groupid']]['name']
+					$db_groups[$group_prototype['groupid']]['name']
 				)
 			);
 		}
@@ -515,23 +817,18 @@ class CHostGroup extends CApiService {
 
 		$this->verifyHostsAndTemplatesAreUnlinkable($hosts_to_unlink, $templates_to_unlink, $groupids);
 
-		$dbScripts = API::Script()->get([
-			'groupids' => $groupids,
-			'output' => ['scriptid', 'groupid'],
-			'nopermissions' => true
+		$db_scripts = DB::select('scripts', [
+			'output' => ['groupid'],
+			'filter' => ['groupid' => $groupids],
+			'limit' => 1
 		]);
 
-		if (!empty($dbScripts)) {
-			foreach ($dbScripts as $script) {
-				if ($script['groupid'] == 0) {
-					continue;
-				}
-				self::exception(ZBX_API_ERROR_PARAMETERS,
-					_s('Host group "%1$s" cannot be deleted, because it is used in a global script.',
-						$db_groups[$script['groupid']]['name']
-					)
-				);
-			}
+		if ($db_scripts) {
+			self::exception(ZBX_API_ERROR_PARAMETERS,
+				_s('Host group "%1$s" cannot be deleted, because it is used in a global script.',
+					$db_groups[$db_scripts[0]['groupid']]['name']
+				)
+			);
 		}
 
 		$corr_condition_group = DBFetch(DBselect(
@@ -549,154 +846,7 @@ class CHostGroup extends CApiService {
 			);
 		}
 
-		// delete screens items
-		$resources = [
-			SCREEN_RESOURCE_HOSTGROUP_TRIGGERS,
-			SCREEN_RESOURCE_HOST_INFO,
-			SCREEN_RESOURCE_TRIGGER_INFO,
-			SCREEN_RESOURCE_TRIGGER_OVERVIEW,
-			SCREEN_RESOURCE_DATA_OVERVIEW
-		];
-		DB::delete('screens_items', [
-			'resourceid' => $groupids,
-			'resourcetype' => $resources
-		]);
-
-		// delete sysmap element
-		if (!empty($groupids)) {
-			DB::delete('sysmaps_elements', ['elementtype' => SYSMAP_ELEMENT_TYPE_HOST_GROUP, 'elementid' => $groupids]);
-		}
-
-		// disable actions
-		// actions from conditions
-		$actionids = [];
-		$dbActions = DBselect(
-			'SELECT DISTINCT c.actionid'.
-			' FROM conditions c'.
-			' WHERE c.conditiontype='.CONDITION_TYPE_HOST_GROUP.
-				' AND '.dbConditionString('c.value', $groupids)
-		);
-		while ($dbAction = DBfetch($dbActions)) {
-			$actionids[$dbAction['actionid']] = $dbAction['actionid'];
-		}
-
-		// actions from operations
-		$dbActions = DBselect(
-			'SELECT o.actionid'.
-			' FROM operations o,opgroup og'.
-			' WHERE o.operationid=og.operationid AND '.dbConditionInt('og.groupid', $groupids).
-			' UNION'.
-			' SELECT o.actionid'.
-			' FROM operations o,opcommand_grp ocg'.
-			' WHERE o.operationid=ocg.operationid AND '.dbConditionInt('ocg.groupid', $groupids)
-		);
-		while ($dbAction = DBfetch($dbActions)) {
-			$actionids[$dbAction['actionid']] = $dbAction['actionid'];
-		}
-
-		if (!empty($actionids)) {
-			$update = [];
-			$update[] = [
-				'values' => ['status' => ACTION_STATUS_DISABLED],
-				'where' => ['actionid' => $actionids]
-			];
-			DB::update('actions', $update);
-		}
-
-		// delete action conditions
-		DB::delete('conditions', [
-			'conditiontype' => CONDITION_TYPE_HOST_GROUP,
-			'value' => $groupids
-		]);
-
-		// delete action operation groups
-		$operationids = [];
-		$dbOperations = DBselect(
-			'SELECT DISTINCT og.operationid'.
-			' FROM opgroup og'.
-			' WHERE '.dbConditionInt('og.groupid', $groupids)
-		);
-		while ($dbOperation = DBfetch($dbOperations)) {
-			$operationids[$dbOperation['operationid']] = $dbOperation['operationid'];
-		}
-		DB::delete('opgroup', [
-			'groupid' => $groupids
-		]);
-
-		// delete action operation commands
-		$dbOperations = DBselect(
-			'SELECT DISTINCT ocg.operationid'.
-			' FROM opcommand_grp ocg'.
-			' WHERE '.dbConditionInt('ocg.groupid', $groupids)
-		);
-		while ($dbOperation = DBfetch($dbOperations)) {
-			$operationids[$dbOperation['operationid']] = $dbOperation['operationid'];
-		}
-		DB::delete('opcommand_grp', [
-			'groupid' => $groupids
-		]);
-
-		// delete empty operations
-		$delOperationids = [];
-		$dbOperations = DBselect(
-			'SELECT DISTINCT o.operationid'.
-			' FROM operations o'.
-			' WHERE '.dbConditionInt('o.operationid', $operationids).
-				' AND NOT EXISTS (SELECT NULL FROM opgroup og WHERE o.operationid=og.operationid)'.
-				' AND NOT EXISTS (SELECT NULL FROM opcommand_grp ocg WHERE o.operationid=ocg.operationid)'
-		);
-		while ($dbOperation = DBfetch($dbOperations)) {
-			$delOperationids[$dbOperation['operationid']] = $dbOperation['operationid'];
-		}
-
-		DB::delete('operations', ['operationid' => $delOperationids]);
-
-		DB::delete('groups', ['groupid' => $groupids]);
-
-		$this->addAuditBulk(AUDIT_ACTION_DELETE, AUDIT_RESOURCE_HOST_GROUP, $db_groups);
-
-		return ['groupids' => $groupids];
-	}
-
-	/**
-	 * Check for duplicated host groups.
-	 *
-	 * @param array  $names
-	 *
-	 * @throws APIException  if host group already exists.
-	 */
-	private function checkDuplicates(array $names) {
-		$db_groups = DB::select('groups', [
-			'output' => ['name'],
-			'filter' => ['name' => $names],
-			'limit' => 1
-		]);
-
-		if ($db_groups) {
-			self::exception(ZBX_API_ERROR_PARAMETERS,
-				_s('Host group "%1$s" already exists.', $db_groups[0]['name'])
-			);
-		}
-	}
-
-	/**
-	 * @param array $groups
-	 *
-	 * @throws APIException if the input is invalid.
-	 */
-	protected function validateCreate(array &$groups) {
-		if (self::$userData['type'] != USER_TYPE_SUPER_ADMIN) {
-			self::exception(ZBX_API_ERROR_PERMISSIONS, _('Only Super Admins can create host groups.'));
-		}
-
-		$api_input_rules = ['type' => API_OBJECTS, 'flags' => API_NOT_EMPTY | API_NORMALIZE, 'uniq' => [['name']], 'fields' => [
-			'name' =>	['type' => API_HG_NAME, 'flags' => API_REQUIRED, 'length' => DB::getFieldLength('groups', 'name')]
-		]];
-		if (!CApiInputValidator::validate($api_input_rules, $groups, '/', $error)) {
-			self::exception(ZBX_API_ERROR_PARAMETERS, $error);
-		}
-
-		$this->checkDuplicates(zbx_objectValues($groups, 'name'));
+		$this->validateDeleteCheckMaintenances($groupids);
 	}
 
 	/**
@@ -708,7 +858,7 @@ class CHostGroup extends CApiService {
 	protected function validateUpdate(array &$groups, array &$db_groups = null) {
 		$api_input_rules = ['type' => API_OBJECTS, 'flags' => API_NOT_EMPTY | API_NORMALIZE, 'uniq' => [['groupid'], ['name']], 'fields' => [
 			'groupid' =>	['type' => API_ID, 'flags' => API_REQUIRED],
-			'name' =>		['type' => API_HG_NAME, 'length' => DB::getFieldLength('groups', 'name')]
+			'name' =>		['type' => API_HG_NAME, 'length' => DB::getFieldLength('hstgrp', 'name')]
 		]];
 		if (!CApiInputValidator::validate($api_input_rules, $groups, '/', $error)) {
 			self::exception(ZBX_API_ERROR_PARAMETERS, $error);
@@ -1278,12 +1428,9 @@ class CHostGroup extends CApiService {
 				]);
 				$hosts = zbx_toHash($hosts, 'groupid');
 				foreach ($result as $groupid => $group) {
-					if (isset($hosts[$groupid])) {
-						$result[$groupid]['hosts'] = $hosts[$groupid]['rowscount'];
-					}
-					else {
-						$result[$groupid]['hosts'] = 0;
-					}
+					$result[$groupid]['hosts'] = array_key_exists($groupid, $hosts)
+						? $hosts[$groupid]['rowscount']
+						: '0';
 				}
 			}
 		}
@@ -1310,12 +1457,9 @@ class CHostGroup extends CApiService {
 				]);
 				$hosts = zbx_toHash($hosts, 'groupid');
 				foreach ($result as $groupid => $group) {
-					if (isset($hosts[$groupid])) {
-						$result[$groupid]['templates'] = $hosts[$groupid]['rowscount'];
-					}
-					else {
-						$result[$groupid]['templates'] = 0;
-					}
+					$result[$groupid]['templates'] = array_key_exists($groupid, $hosts)
+						? $hosts[$groupid]['rowscount']
+						: '0';
 				}
 			}
 		}
@@ -1402,4 +1546,38 @@ class CHostGroup extends CApiService {
 			}
 		}
 	}
+
+	/**
+	 * Validates if host groups may be deleted, due to maintenance constrain.
+	 *
+	 * @throws APIException if a constrain failed
+	 *
+	 * @param array $groupids
+	 */
+	protected function validateDeleteCheckMaintenances(array $groupids) {
+		$maintenance = DBfetch(DBselect(
+			'SELECT m.name'.
+			' FROM maintenances m'.
+			' WHERE NOT EXISTS ('.
+				'SELECT NULL'.
+				' FROM maintenances_groups mg'.
+				' WHERE m.maintenanceid=mg.maintenanceid'.
+					' AND '.dbConditionInt('mg.groupid', $groupids, true).
+			')'.
+				' AND NOT EXISTS ('.
+					'SELECT NULL'.
+					' FROM maintenances_hosts mh'.
+					' WHERE m.maintenanceid=mh.maintenanceid'.
+				')'
+		));
+
+		if ($maintenance) {
+			self::exception(ZBX_API_ERROR_PARAMETERS, _n(
+				'Cannot delete host group because maintenance "%1$s" must contain at least one host or host group.',
+				'Cannot delete selected host groups because maintenance "%1$s" must contain at least one host or host group.',
+				$maintenance['name'],
+				count($groupids)
+			));
+		}
+	}
 }
